import { Logger } from "./Logger.js";
/*
Each State should adhere to this interface:

export class ExampleState {
	constructor() { }
	async start() { }
	async end() { }
	update(delta) { }
	draw(context) { }
}

*/
export class FiniteStateMachine {
	constructor() {
		this.states = {};
		this.currentState = null;
	}

	addState(key, state) {
		this.states[key] = state;
	}

	setCurrentState(key) {
		this.currentState = key;
	}

	getState(key) {
		return this.states[key];
	}

	async initialLoad(defaultKey) {
		this.transitionTo(defaultKey);
	}

	async transitionTo(key, addToHistory = true) {
		let lastState = this.getCurrentState();

		if (lastState) {
			await lastState.end(this.getState(key));
		}
		if (addToHistory) {
			history.pushState({}, key, "?s=" + key);
		}
		this.setCurrentState(key);
		await this.getState(key).start(lastState);

	}

	updateCurrentState(delta) {
		this.getCurrentState().update(delta);
	}

	drawCurrentState(context) {
		this.getCurrentState().draw(context);
	}

	getCurrentState() {
		return this.getState(this.currentState);
	}

	currentStateHas(functionName) {
		return typeof this.getCurrentState()[functionName] == 'function';
	}

	onResize = () => {
		if (!this.currentStateHas('onResize')) {
			Logger.warn("no handler for onResize", this.currentState);
			return;
		}
		this.getCurrentState().onResize();
	}
	onMouseMove = (event) => {
		if (!this.currentStateHas('onMouseMove')) {
			Logger.warn("no handler for onMouseMove", this.currentState);
			return;
		}
		this.getCurrentState().onMouseMove(event);
	}
	onMouseDown = (event) => {
		if (!this.currentStateHas('onMouseDown')) {
			Logger.warn("no handler for onMouseDown", this.currentState);
			return;
		}
		this.getCurrentState().onMouseDown(event);
	}
	onMouseUp = (event) => {
		if (!this.currentStateHas('onMouseUp')) {
			Logger.warn("no handler for onMouseUp", this.currentState);
			return;
		}
		this.getCurrentState().onMouseUp(event);
	}
	onRightClick = (event) => {
		if (!this.currentStateHas('onRightClick')) {
			Logger.warn("no handler for onRightClick", this.currentState);
			return;
		}
		this.getCurrentState().onRightClick(event);
	}
	onTouchStart = (event) => {
		if (!this.currentStateHas('onTouchStart')) {
			Logger.warn("no handler for onTouchStart", this.currentState);
			return;
		}
		this.getCurrentState().onTouchStart(event);
	}
	onTouchMove = (event) => {
		if (!this.currentStateHas('onTouchMove')) {
			Logger.warn("no handler for onTouchMove", this.currentState);
			return;
		}
		this.getCurrentState().onTouchMove(event);
	}
	onTouchEnd = (event) => {
		if (!this.currentStateHas('onTouchEnd')) {
			Logger.warn("no handler for onTouchEnd", this.currentState);
			return;
		}
		this.getCurrentState().onTouchEnd(event);
	}

	onPopState = (event) => {
		let location = event.target.location;
		let searchParams = new URLSearchParams(location.search);
		let lastState = searchParams.get("s");
		if (lastState) {
			this.transitionTo(lastState, false);
		}
	}

	popState = () => {
		window.history.back();
	}

	websocketConnect = (websocket, event) => {
		if (!this.currentStateHas('websocketConnect')) {
			Logger.warn("no handler for websocketConnect", this.currentState);
			return;
		}
		this.getCurrentState().websocketConnect(websocket, event);
	}

	websocketMessage = (websocket, event) => {
		if (!this.currentStateHas('websocketMessage')) {
			Logger.warn("no handler for websocketMessage", this.currentState);
			return;
		}
		this.getCurrentState().websocketMessage(websocket, event);
	}

	websocketError = (websocket, event) => {
		if (!this.currentStateHas('websocketError')) {
			Logger.warn("no handler for websocketError", this.currentState);
			return;
		}
		this.getCurrentState().websocketError(websocket, event);
	}

	websocketClose = (websocket, event) => {
		if (!this.currentStateHas('websocketClose')) {
			Logger.warn("no handler for websocketClose", this.currentState);
			return;
		}
		this.getCurrentState().websocketClose(websocket, event);
	}
	
}
