import { canvas, fsm, StateKey } from '../Core.js';
import { keys, KeyCode } from '../KeyboardInput.js';
import { Acquire } from '../Acquire.js';

export class ExploreMapState {
	constructor() {
		this.camera = { x: -128, y: -128 };
		this.loader = new Acquire('./templates');
		this.map = { width: 16384, height: 16384 };
		this.isDragging = false;
		this.originalSector = { x: 0, y: 0 };
		this.destinationSector = { x: 0, y: 0 };
		this.lineDashOffset = 0;
		this.distance = 0;
	}

	init() {
		this.camera = { x: -140, y: -140 };
		this.isDragging = false;
		this.originalSector = { x: 0, y: 0 };
		this.destinationSector = { x: 0, y: 0 };
		this.lineDashOffset = 0;
	}

	async start(fromState) {
		this.init();
		canvas.style.opacity = 1;
		this.navMenu = await this.loader.loadDOM('mapnav.html');
		this.navMenu.addEventListener("mouseup", e => e.stopImmediatePropagation());
		this.navMenu.addEventListener("touchend", e => e.stopImmediatePropagation());

		this.navMenu.getElementsByClassName('nav-back')[0].addEventListener('click', () => {
			this.destinationSector.x = this.originalSector.x;
			this.destinationSector.y = this.originalSector.y;
			fsm.popState();
		});

		this.navMenu.getElementsByClassName('nav-jump')[0].addEventListener('click', this.handleJump);

		document.body.appendChild(this.navMenu);
		this.navMenu.style.opacity = 1;
		this.originalSector.x = fromState.currentSector.x;
		this.originalSector.y = fromState.currentSector.y;

		this.destinationSector.x = this.originalSector.x;
		this.destinationSector.y = this.originalSector.y;

		let canvasBounds = canvas.getBoundingClientRect();

		this.camera.x = (this.originalSector.x * 128) + 64 - (canvasBounds.width / 2);
		this.camera.y = (this.originalSector.y * 128) + 64 - (canvasBounds.height / 2);
	}

	handleJump = (event) => {
		event.preventDefault();
		if (this.distance > 0 && this.distance < 2.5) {
			alert("Jumping to " + this.destinationSector.x + "," + this.destinationSector.y);
			fsm.transitionTo(StateKey.Game);
		} else {
			alert("Please choose a valid jump destination");
		}
	}

	async end() {
		document.body.removeChild(this.navMenu);
	}

	update(delta) {
		if (keys[KeyCode.Escape]) {
			fsm.popState();
			delete (keys[KeyCode.Escape]);
			return;
		}

		this.lineDashOffset++;
		this.lineDashOffset %= 31;

		this.distance = Math.hypot(this.originalSector.x - this.destinationSector.x, this.originalSector.y - this.destinationSector.y);
	}

	draw(context) {
		let canvasBounds = canvas.getBoundingClientRect();
		context.fillStyle = "black";
		context.beginPath();
		context.rect(0, 0, canvasBounds.width, canvasBounds.height);
		context.fill();

		context.save();
		context.translate(-this.camera.x, -this.camera.y);

		context.strokeStyle = "limegreen";
		context.fillStyle = "white";
		context.font = "1em 'Syne Mono', serif";
		context.textAlign = "left";

		for (let y = -16; y < 16; y++) {
			for (let x = -16; x < 16; x++) {
				context.save();
				context.translate(x * 128, y * 128);
				context.beginPath();
				context.rect(0, 0, 128, 128);
				context.stroke();


				context.fillText(`${x},${y}`, 5, 15);
				context.restore();
			}
		}

		context.strokeStyle = "yellow";
		context.fillStyle = "yellow";
		if (this.distance > 2.5) {
			context.strokeStyle = "crimson";
			context.fillStyle = "crimson";
		}
		context.lineWidth = 4;

		context.save();
			context.translate(this.originalSector.x * 128, this.originalSector.y * 128);
			context.beginPath();
			context.rect(0, 0, 128, 128);
			context.stroke();

			context.fillText(`${this.originalSector.x},${this.originalSector.y}`, 5, 15);
			context.restore();

		if (this.distance) {
			context.save();
			context.translate(this.destinationSector.x * 128, this.destinationSector.y * 128);
			context.beginPath();
			context.rect(0, 0, 128, 128);
			context.stroke();

			context.fillText(`${this.destinationSector.x},${this.destinationSector.y}`, 5, 15);
			context.restore();

			context.beginPath();
			context.setLineDash([20, 10]);
			context.lineDashOffset = -this.lineDashOffset;
			context.moveTo(this.originalSector.x * 128 + 64, this.originalSector.y * 128 + 64);
			context.lineTo(this.destinationSector.x * 128 + 64, this.destinationSector.y * 128 + 64);
			context.stroke();
			context.setLineDash([0]);
		}

		context.restore();

		context.font = "2em 'Syne Mono', serif";
		context.textAlign = "center";
		context.fillStyle = "white";
		context.fillText("Sector Alpha", canvasBounds.width / 2, 50);
	}

	onResize() {
		let canvasBounds = canvas.getBoundingClientRect();
		this.camera.center.x = canvasBounds.width / 2;
		this.camera.center.y = canvasBounds.height / 2;
	}

	onMouseDown(event) {
		if (event.buttons == 1 && event.button == 0) {
			this.dragStart = {
				x: event.clientX,
				y: event.clientY,
			};
		}
	}

	onMouseMove(event) {
		if (event.buttons == 1 && event.button == 0) {
			let deltaX = this.dragStart.x - event.clientX;
		let deltaY = this.dragStart.y - event.clientY;
		this.camera.x += deltaX;
		this.camera.y += deltaY;

		if (deltaX != 0 || deltaY != 0) {
			this.isDragging = true;
		}
		this.dragStart.x = event.clientX;
		this.dragStart.y = event.clientY;
		}
	}

	onMouseUp(event) {
		if (!this.isDragging) {
			this.destinationSector.x = Math.floor((event.clientX + this.camera.x) / 128);
			this.destinationSector.y = Math.floor((event.clientY + this.camera.y) / 128);
		}
		this.dragStart = null;
		this.isDragging = false;
	}

	onTouchStart(event) {
		this.dragStart = {
			x: event.touches[0].clientX,
			y: event.touches[0].clientY,
		};
	}

	onTouchMove(event) {
		let deltaX = this.dragStart.x - event.touches[0].clientX;
		let deltaY = this.dragStart.y - event.touches[0].clientY;
		this.camera.x += deltaX;
		this.camera.y += deltaY;

		if (deltaX != 0 || deltaY != 0) {
			this.isDragging = true;
		}
		this.dragStart.x = event.touches[0].clientX;
		this.dragStart.y = event.touches[0].clientY;
	}

	onTouchEnd(event) {
		if (!this.isDragging) {
			this.destinationSector.x = Math.floor((event.changedTouches[0].clientX + this.camera.x) / 128);
			this.destinationSector.y = Math.floor((event.changedTouches[0].clientY + this.camera.y) / 128);
		}
		this.dragStart = null;
		this.isDragging = false;
	}
}
