import { canvas, fsm, StateKey, webSocket, websocketReconnect } from '../Core.js';
import { keys, KeyCode } from '../KeyboardInput.js';
import { Ship } from '../canvas-elements/Ship.js';
import { Logger } from '../Logger.js';
import { GameWebsocketCommandHandler } from '../GameWebsocketCommandHandler.js';
import { Room } from '../canvas-elements/Room.js';
import { PlayerController } from '../components/PlayerController.js';
import { DomUiManager } from '../DomUiManager.js';
import { Acquire } from '../Acquire.js';

export class GameState {
	constructor() {
		this.debug = false;
		this.camera = { x: 5000, y: 5000, center: { x: 0, y: 0 } };
		this.currentSector = { x: 0, y: 0 };
		this.room = new Room(this.camera, this.currentSector);
		this.ship = new Ship(this.room.size.width / 2, this.room.size.height / 2);
		this.playerController = new PlayerController(this.ship, this.camera);

		this.navMenu = new DomUiManager('./templates');
		this.shipInfo = new DomUiManager('./templates');
		this.socketHandler = new GameWebsocketCommandHandler();
		this.moveDirty = false;
		this.moveDirtyIntervalId = 0;
		this.acquire = new Acquire('');
	}

	init() {
		this.camera.x = this.room.size.width / 2;
		this.camera.y = this.room.size.height / 2;
		this.room.init();
		this.ship.init(5, this.room.size.width / 2, this.room.size.height / 2);
		webSocket.send(`connect`);
		webSocket.send(`join ${this.currentSector.x} ${this.currentSector.y}`);
		this.moveDirtyIntervalId = setInterval(() => { if (this.moveDirty) { this.sendPosition(); } this.moveDirty = false; }, 300);
	}

	async start(stateFrom) {
		canvas.style.opacity = 1;
		await this.navMenu.load('navmenu.html');
		this.navMenu.muteCanvas();

		this.navMenu.getChildById('nav-menu-info').addEventListener('click', e => {
			let dataToUpdate = { "ship-color": this.ship.color };
			this.shipInfo.toggle(e, dataToUpdate);
		});
		this.navMenu.getChildById('nav-menu-explore').addEventListener('click', this.handleExploreClick);

		await this.shipInfo.load('shipinfo.html');
		this.shipInfo.getChildrenByClassName('close')[0].addEventListener('click', e => this.shipInfo.hide(e));
		this.shipInfo.getChildById("ship-color").addEventListener('change', e => this.shipColorChange(e));
		this.shipInfo.muteCanvas();

		if (stateFrom.destinationSector) {
			this.currentSector.x = stateFrom.destinationSector.x;
			this.currentSector.y = stateFrom.destinationSector.y;
		}

		this.init();
	}

	muteCanvas(domElement) {
		domElement.addEventListener("touchstart", e => e.stopImmediatePropagation());
		domElement.addEventListener("touchmove", e => e.stopImmediatePropagation());
		domElement.addEventListener("touchend", e => e.stopImmediatePropagation());
		domElement.addEventListener("mouseup", e => e.stopImmediatePropagation());
		domElement.addEventListener("mousemove", e => e.stopImmediatePropagation());
		domElement.addEventListener("mousedown", e => e.stopImmediatePropagation());
	}

	handleExploreClick = event => {
		event.preventDefault();
		fsm.transitionTo(StateKey.Map);
	};

	async end() {
		this.shipInfo.remove();
		this.navMenu.remove();
		clearInterval(this.moveDirtyIntervalId);
	}

	update(delta) {
		let canvasBounds = canvas.getBoundingClientRect();
		if (keys[KeyCode.Escape]) {
			fsm.popState();
			delete (keys[KeyCode.Escape]);
			return;
		}

		if (keys[KeyCode.F3]) {
			this.debug = !this.debug;
			this.ship.debug = this.debug;
			keys[KeyCode.F3] = false;

		}

		this.room.update(delta);
		this.ship.update(delta);

		// if (this.ship.position.x + (this.ship.width / 2) <= 0) {
		// 	this.ship.position.x = this.room.size.width - (this.ship.width / 2);
		// }
		// if (this.ship.position.y + (this.ship.height / 2) <= 0) {
		// 	this.ship.position.y = this.room.size.height - (this.ship.height / 2);
		// }
		// if (this.ship.position.x - (this.ship.width / 2) >= this.room.size.width) {
		// 	this.ship.position.x = (this.ship.width / 2);
		// }
		// if (this.ship.position.y - (this.ship.height / 2) > this.room.size.height) {
		// 	this.ship.position.y = (this.ship.height / 2);
		// }

		if (this.ship.position.x - this.camera.x < -canvasBounds.width * 0.25) {
			this.camera.x = this.ship.position.x + canvasBounds.width * 0.25;
		}

		if (this.ship.position.x - this.camera.x > canvasBounds.width * 0.25) {
			this.camera.x = this.ship.position.x - canvasBounds.width * 0.25;
		}

		if (this.ship.position.y - this.camera.y < -canvasBounds.height * 0.25) {
			this.camera.y = this.ship.position.y + canvasBounds.height * 0.25;
		}

		if (this.ship.position.y - this.camera.y > canvasBounds.height * 0.25) {
			this.camera.y = this.ship.position.y - canvasBounds.height * 0.25;
		}
	}

	draw(context) {
		let canvasBounds = canvas.getBoundingClientRect();
		context.fillStyle = "black";
		context.beginPath();
		context.rect(0, 0, canvasBounds.width, canvasBounds.height);
		context.fill();


		context.save();
		context.translate(-this.camera.x + canvasBounds.width / 2, -this.camera.y + canvasBounds.height / 2);

		this.room.draw(context);
		this.ship.draw(context);

		context.restore();
	}

	onResize() {
	}

	onMouseDown(event) {
		this.playerController.onMouseDown(event);
		this.sendPosition();
	}

	onMouseMove(event) {
		this.playerController.onMouseMove(event);
		this.moveDirty = true;
	}

	onMouseUp(event) {
		this.playerController.onMouseUp(event);
		this.sendPosition();
	}

	onTouchStart(event) {
		this.playerController.onTouchStart(event);
		this.sendPosition();
	}

	onTouchMove(event) {
		this.playerController.onTouchMove(event);
		this.moveDirty = true;
	}

	onTouchEnd(event) {
		this.playerController.onTouchEnd(event);
		this.sendPosition();
	}

	sendPosition() {
		let moveString = `move ${this.ship.position.x} ${this.ship.position.y} ${this.ship.angle} ${this.ship.velocity.x} ${this.ship.velocity.y} ${this.ship.isAccelerating()}`;
		webSocket.send(moveString);
	}

	updatePosition() {
		this.sendPosition();
	}

	websocketConnect(websocket, event) {
		websocket.send("connect");
	}

	websocketMessage(websocket, event) {
		if (!this.socketHandler) {
			Logger.warn("socket handler not set up yet", event);
			return;
		}
		let action = this.socketHandler.parse(event.data);
		action(this);
	}

	websocketClose(websocket, event) {
		Logger.info("game disconnected:", event);
		setTimeout(websocketReconnect, 5000);
	}

	clearPopulation() {
		this.room.clearPopulation();
	}

	addOtherShip(otherShipData) {
		this.room.addOtherShip(otherShipData);
	}

	updateOtherShip(otherShipData) {
		this.room.updateOtherShip(otherShipData);
	}

	removeOtherShip(shipId) {
		this.room.removeOtherShip(shipId);
	}

	shipColorChange(event) {
		//TODO: prevent invalid colors
		this.ship.color = event.target.value;
		this.acquire.apiPut(`./api/ship/${this.ship.id}/color/${event.target.value}`);
	}

	connectPlayer(playerId) {
		this.ship.id = playerId;
	}
}
