import { canvas, fsm, StateKey, websocketReconnect } from '../Core.js';
import { Starfield } from '../canvas-elements/Starfield.js';
import { Logger } from '../Logger.js';
import { Room } from '../canvas-elements/Room.js';
import { DomUiManager } from '../DomUiManager.js';
import { GameWebsocketCommandHandler } from '../GameWebsocketCommandHandler.js';

export class MenuState {
	constructor() {
		this.starfield = Starfield.GetInstance();
		this.camera = { x: 5000, y: 5000, center: { x: 0, y: 0 } };
		this.currentSector = { x: 0, y: 0 };
		this.room = new Room(this.camera, this.currentSector);
		this.menuUI = new DomUiManager('./templates');
		this.socketHandler = new GameWebsocketCommandHandler();
	}
	async start() {
		canvas.style.opacity = 1;
		await this.menuUI.load('mainmenu.html');

		this.menuUI.getChildById('main-menu-start').addEventListener('click', () => {
			fsm.transitionTo(StateKey.Game);
		});

		this.init();
	}
	async end() {
		this.menuUI.remove();
	}

	init() {
		// this.leave();
		this.camera.x = this.room.size.width / 2;
		this.camera.y = this.room.size.height / 2;
		this.room.init();
	}
	update(delta) {
		let canvasBounds = canvas.getBoundingClientRect();
		this.room.update(delta);
		let frequency = 0.05;
		let time = (new Date().getTime() / 1000) % (((1 / frequency) * 2) * Math.PI);
		let radius = 1000;
		this.camera.x = radius * Math.cos(frequency * time) + (this.room.size.width / 2);
		this.camera.y = radius * Math.sin(frequency * time) + (this.room.size.height / 2) + radius;
	}
	
	draw(context) {
		let canvasBounds = canvas.getBoundingClientRect();
		context.fillStyle = "black";
		context.beginPath();
		context.rect(0, 0, canvasBounds.width, canvasBounds.height);
		context.fill();

		context.save();
		context.translate(-this.camera.x + canvasBounds.width / 2, -this.camera.y + canvasBounds.height / 2);
		this.room.draw(context);
		context.restore();
	}

	websocketConnect(websocket, event) {
		websocket.send("connect");
	}

	websocketMessage(websocket, event) {
		if (!this.socketHandler) {
			Logger.warn("socket handler not set up yet", event);
			return;
		}
		this.socketHandler.parse(event.data)(this);
	}

	websocketClose(websocket, event) {
		Logger.info("menu disconnected:", event);
		setTimeout(websocketReconnect, 5000);
	}

	addOtherShip(otherShipData) {
		this.room.addOtherShip(otherShipData);
	}

	updateOtherShip(otherShipData) {
		this.room.updateOtherShip(otherShipData);
	}

	removeOtherShip(shipId) {
		this.room.removeOtherShip(shipId);
	}

	leave() {
		websocket.send("leave");
	}
}
